## File "PloneInstaller/Scripts/actions_setup.py"
## -*- coding: utf-8 -*-
# 
# This file use the configuration described in "Configs/actions.py" to setup the portal actions.



from Products.CMFCore.utils import getToolByName



#########################################################################################################
########################################## Public methods ###############################################
#########################################################################################################



def actions_install(self, out, project_name, globals):
    try:
      actions_module = __import__("Configs.actions",
                                            globals, locals(),
                                           [
                                              'DELETED_ACTIONS',
                                              'CREATED_ACTIONS',
                                              'UPDATED_ACTIONS',
                                            ]
                                          )

      out.openSection("Configs.actions setup")

      # Deleted actions configuration
      deleted_actions = getattr(actions_module, 'DELETED_ACTIONS', None)
      if deleted_actions != None:
        _installDeletedActions(self, out, deleted_actions)

      # Created actions configuration
      created_actions = getattr(actions_module, 'CREATED_ACTIONS', None)
      if created_actions != None:
        _installCreatedActions(self, out, created_actions)

      # Updated actions configuration
      updated_actions = getattr(actions_module, 'UPDATED_ACTIONS', None)
      if updated_actions != None:
        _installUpdatedActions(self, out, updated_actions)

      out.closeSection("Configs.actions setup")

    except ImportError:
        out.info("Configs.actions setup : Noting to be done.")



##---------------------------------------------------------------------------



def actions_remove(self, out, project_name, globals):
    try:
      actions_module = __import__("Configs.actions",
                                            globals, locals(),
                                           [
                                              'DELETED_ACTIONS',
                                              'CREATED_ACTIONS',
                                              'UPDATED_ACTIONS',
                                            ]
                                          )

      out.openSection("Configs.actions removal")

      # Deleted actions configuration
      deleted_actions = getattr(actions_module, 'DELETED_ACTIONS', None)
      if deleted_actions != None:
        _removeDeletedActions(self, out, deleted_actions)

      # Created actions configuration
      created_actions = getattr(actions_module, 'CREATED_ACTIONS', None)
      if created_actions != None:
        _removeCreatedActions(self, out, created_actions)

      # Updated actions configuration
      updated_actions = getattr(actions_module, 'UPDATED_ACTIONS', None)
      if updated_actions != None:
        _removeUpdatedActions(self, out, updated_actions)

      out.closeSection("Configs.actions removal")

    except ImportError:
        out.info("Configs.actions removal : Noting to be done.")



#########################################################################################################
######################################## Install functions ##############################################
#########################################################################################################



## _installDeletedActions
# 
# Setup Deleted actions configuration
def _installDeletedActions(self, out, deleted_actions):

    errorFlag = False

    out.openSubsection("Deleted actions setup")

    for object_name, object_actions in deleted_actions.items(): # For each object to be modified

        try: # Check for object existance
            portal = self.portal_url.getPortalObject()
            object = portal.restrictedTraverse(object_name)
        except AttributeError, e:
            errorFlag = True
            out.error("object '%s' not found. (Error message : %s)" % (object_name, str(e)) )
        else:

            # We get the indices of deleted actions.
            existing_actions = list(object.listActions())
            deleted_actions_indices = []
            for action_description in object_actions: # For each action to be updated in the current object.
                actionFound = False
                for action in existing_actions:
                    if action_description.get('id') == action.getId() and action_description.get('category') == action.getCategory():
                        actionFound = True
                        deleted_actions_indices.append(existing_actions.index(action))
                if not actionFound:
                    errorFlag = True
                    out.error("action '%s/%s' is no present in object '%s'." % (action_description['category'], action_description['id'], object_name) )

            try: # We delete the actions
                object.deleteActions(selections = deleted_actions_indices)
            except Exception, e:
                errorFlag = True
                out.error("Failed to delete actions from object '%s'. (Error message : %s)" % (object_name, str(e)) )

    if not errorFlag:
        out.success("Completed deleted actions setup without errors.")

    out.closeSubsection("Deleted actions setup")



##---------------------------------------------------------------------------



## _installCreatedActions
# 
# Setup created actions configuration
def _installCreatedActions(self, out, created_actions):

    errorFlag = False

    out.openSubsection("Created actions setup")

    for object_name, object_actions in created_actions.items(): # For each object to be modified

        try: # Check for object existance
            portal = self.portal_url.getPortalObject()
            object = portal.restrictedTraverse(object_name)
        except AttributeError, e:
            errorFlag = True
            out.error("object '%s' not found. (Error message : %s)" % (object_name, str(e)) )
        else:

            for action_description in object_actions: # For each action to be updated in the current object.
                try:
                    object.addAction(**action_description)
                    out.log("Created action '%s/%s' in object '%s'." % (action_description['category'], action_description['id'], object_name,) )
                except Exception, e:
                    errorFlag = True
                    out.error("Unable to create action '%s/%s' in object '%s'. (Error message : %s)" % (action_description['category'], action_description['id'], object_name, str(e)) )

    if not errorFlag:
        out.success("Completed created actions setup without errors.")

    out.closeSubsection("Created actions setup")



##---------------------------------------------------------------------------



## _installUpdatedActions
# 
# Setup updated actions configuration
def _installUpdatedActions(self, out, updated_actions):

    errorFlag = False

    out.openSubsection("Updated actions setup")

    for object_name, object_actions in updated_actions.items(): # For each object to be modified

        try: # Check for object existance
            portal = self.portal_url.getPortalObject()
            object = portal.restrictedTraverse(object_name)
        except AttributeError, e:
            errorFlag = True
            out.error("object '%s' not found. (Error message : %s)" % (object_name, str(e)) )
        else:

            for action_description in object_actions: # For each action to be updated in the current object.
                action = object.getActionObject('%s/%s' % (action_description['category'], action_description['id']) )
                if action :
                    action_description['id'] = action_description.get('new_id', action_description['id'])
                    action_description['category'] = action_description.get('new_category', action_description['category'])
                    if action_description.has_key('new_id'):
                        del action_description['new_id']
                    if action_description.has_key('new_category'):
                        del action_description['new_category']
                    action.edit(**action_description)
                    out.log("Updated action '%s/%s' in object '%s'." % (action_description['category'], action_description['id'], object_name,) )
                else :
                    errorFlag = True
                    out.error("action '%s/%s' is no present in object '%s'." % (action_description['category'], action_description['id'], object_name) )

    if not errorFlag:
        out.success("Completed updated actions setup without errors.")

    out.closeSubsection("Updated actions setup")



#########################################################################################################
######################################### Remove functions ##############################################
#########################################################################################################



## _removeDeletedActions
# 
# Remove deleted actions configuration.
# TODO : Create a remove deleted actions method.
def _removeDeletedActions(self, out, deleted_actions):

    out.openSubsection("Deleted actions removal")

    for object_name, object_actions in deleted_actions.items(): # For each object to be modified
        try: # Check for object existance
            portal = self.portal_url.getPortalObject()
            object = portal.restrictedTraverse(object_name)
        except AttributeError, e:
            errorFlag = True
            out.error("object '%s' not found. (Error message : %s)" % (object_name, str(e)) )

    out.info("Deleted actions configuration can not be removed.")

    out.closeSubsection("Deleted actions removal")



##---------------------------------------------------------------------------



## _removeCreatedActions
# 
# Remove created actions configuration.
# TODO : Create a remove created actions method.
def _removeCreatedActions(self, out, created_actions):
    errorFlag = False

    out.openSubsection("Created actions removal")

    for object_name, object_actions in created_actions.items(): # For each object to be modified

        try: # Check for object existance
            portal = self.portal_url.getPortalObject()
            object = portal.restrictedTraverse(object_name)
        except AttributeError, e:
            errorFlag = True
            out.error("object '%s' not found. (Error message : %s)" % (object_name, str(e)) )
        else:

            # We get the indices of deleted actions.
            existing_actions = list(object.listActions())
            deleted_actions_indices = []
            for action_description in object_actions: # For each action to be updated in the current object.
                actionFound = False
                for action in existing_actions:
                    if action_description.get('id') == action.getId() and action_description.get('category') == action.getCategory():
                        actionFound = True
                        deleted_actions_indices.append(existing_actions.index(action))
                if not actionFound:
                    errorFlag = True
                    out.error("action '%s/%s' is no present in object '%s'." % (action_description['category'], action_description['id'], object_name) )

            try: # We delete the actions
                object.deleteActions(selections = deleted_actions_indices)
            except Exception, e:
                errorFlag = True
                out.error("Failed to remove actions from object '%s'. (Error message : %s)" % (object_name, str(e)) )

    if not errorFlag:
        out.success("Completed created actions removal without errors.")

    out.closeSubsection("Created actions removal")



##---------------------------------------------------------------------------



## _removeUpdatedActions
# 
# Remove updated actions configuration.
# TODO : Create a remove updated actions method.
def _removeUpdatedActions(self, out, updated_actions):

    out.openSubsection("Updated actions removal")

    for object_name, object_actions in updated_actions.items(): # For each object to be modified
        try: # Check for object existance
            portal = self.portal_url.getPortalObject()
            object = portal.restrictedTraverse(object_name)
        except AttributeError, e:
            errorFlag = True
            out.error("object '%s' not found. (Error message : %s)" % (object_name, str(e)) )

    out.info("Updated actions configuration can not be removed.")

    out.closeSubsection("Updated actions removal")



#########################################################################################################
######################################### Private functions #############################################
#########################################################################################################



