## File "PloneInstaller/Scripts/external_methods_setup.py"
## -*- coding: utf-8 -*-
# 
# This file use the configuration described in "Configs/external_methods.py" to setup the portal external_methods.

from Products.CMFCore.utils import getToolByName
from Products.ExternalMethod.ExternalMethod import manage_addExternalMethod

#########################################################################################################
########################################## Public methods ###############################################
#########################################################################################################



def external_methods_install(self, out, project_name, globals):
    try:
      external_methods_module = __import__("Configs.external_methods",
                                            globals, locals(),
                                           [
                                              'EXTERNAL_METHODS',
                                            ]
                                          )

      out.openSection("Configs.external_methods setup")

      # ExternalMethods installed configuration
      external_methods = getattr(external_methods_module, 'EXTERNAL_METHODS', None)
      if external_methods != None:
        _installExternalMethods(self, out, project_name, external_methods)

      out.closeSection("Configs.external_methods setup")

    except ImportError:
        out.info("Configs.external_methods setup : Noting to be done.")



##---------------------------------------------------------------------------



def external_methods_remove(self, out, project_name, globals):
    try:
      external_methods_module = __import__("Configs.external_methods",
                                            globals, locals(),
                                           [
                                              'EXTERNAL_METHODS',
                                            ]
                                          )

      out.openSection("Configs.external_methods removal")

      # ExternalMethods installed configuration
      external_methods = getattr(external_methods_module, 'EXTERNAL_METHODS', None)
      if external_methods != None:
        _removeExternalMethods(self, out, external_methods)

      out.closeSection("Configs.external_methods removal")

    except ImportError:
        out.info("Configs.external_methods removal : Noting to be done.")



#########################################################################################################
######################################## Install functions ##############################################
#########################################################################################################



## _installExternalMethods
# 
# Setup external methods by this product
def _installExternalMethods(self, out, project_name, external_methods):
    errorFlag = False

    out.openSubsection("External methods setup")


    try:
        url_tool = getToolByName(self, 'portal_url')
        portal = url_tool.getPortalObject()
    except Exception, e:
        errorFlag = True
        out.error("failed to get url_tool. (Error message : %s)" % (str(e),) )
    else:
        for external_method_description in external_methods:
            try:
                manage_addExternalMethod(portal,external_method_description['id'],
                                          external_method_description['title'],
                                          '%s.%s' % (project_name, external_method_description['module']),
                                          external_method_description['function'])

                out.info("External method '%s' successfully installed." % external_method_description['id'])
            except Exception, e:
                errorFlag = True
                out.error("failed to install external method '%s'. (Error message : %s)" % (external_method_description['id'], str(e)) )

    if not errorFlag:
        out.success("Completed external_methods installed setup without errors.")

    out.closeSubsection("External methods setup")



#########################################################################################################
######################################### Remove functions ##############################################
#########################################################################################################



## _removeExternalMethods
# 
# Remove external methods  configuration.
def _removeExternalMethods(self, out, external_methods):

    out.openSubsection("External methods removal")

    try:
        url_tool = getToolByName(self, 'portal_url')
        portal = url_tool.getPortalObject()
    except Exception, e:
        errorFlag = True
        out.error("failed to get url_tool. (Error message : %s)" % (str(e),) )
    else:
        for external_method_description in external_methods:
            try:
                portal.manage_delObjects(ids = [ external_method_description['id'], ] )

                out.info("External method '%s' successfully removed." % external_method_description['id'])
            except Exception, e:
                errorFlag = True
                out.error("failed to remove external method '%s'. (Error message : %s)" % (external_method_description['id'], str(e)) )

    out.closeSubsection("External methods removal")



#########################################################################################################
######################################### Private functions #############################################
#########################################################################################################

