## File "PloneInstaller/Scripts/security_setup.py"
## -*- coding: utf-8 -*-
# 
# This file use the configuration described in "Configs/security.py" to setup the portal security.



from Products.CMFCore.utils import getToolByName



#########################################################################################################
########################################## Public methods ###############################################
#########################################################################################################



def security_install(self, out, project_name, globals):
    try:
      security_module = __import__("Configs.security",
                                            globals, locals(),
                                           [
                                              'UPDATED_SECURITY',
                                              'ADDED_ROLES',
                                            ]
                                          )

      out.openSection("Configs.security setup")

      # Added roles configuration
      added_roles = getattr(security_module, 'ADDED_ROLES', None)
      if added_roles != None:
        _installAddedRoles(self, out, added_roles)

      # Security configuration
      updated_security = getattr(security_module, 'UPDATED_SECURITY', None)
      if updated_security != None:
        _installUpdatedSecurity(self, out, updated_security)

      out.closeSection("Configs.security setup")

    except ImportError:
        out.info("Configs.security setup : Noting to be done.")



##---------------------------------------------------------------------------



def security_remove(self, out, project_name, globals):
    try:
      security_module = __import__("Configs.security",
                                            globals, locals(),
                                           [
                                              'UPDATED_SECURITY',
                                              'ADDED_ROLES',
                                            ]
                                          )

      out.openSection("Configs.security removal")

      # Security configuration
      updated_security = getattr(security_module, 'UPDATED_SECURITY', None)
      if updated_security != None:
        _removeUpdatedSecurity(self, out, updated_security)

      # Added roles configuration
      added_roles = getattr(security_module, 'ADDED_ROLES', None)
      if added_roles != None:
        _removeAddedRoles(self, out, added_roles)

      out.closeSection("Configs.security removal")

    except ImportError:
        out.info("Configs.security removal : Noting to be done.")



#########################################################################################################
######################################## Install functions ##############################################
#########################################################################################################



## _installAddedRoles
# 
# Setup added roles configuration
def _installAddedRoles(self, out, added_roles):
    errorFlag = False

    out.openSubsection("Added roles setup")

    valid_roles = self.validRoles()

    for role in added_roles:
      if role in valid_roles:
        out.info("Role '%s' is already present." % role)
      else:
        try:
          self.manage_defined_roles(submit='Add Role',
                                    REQUEST={'role': role})
          out.info("Added role '%s'." % role)
        except Exception, e:
          errorFlag = True
          out.error("Failed to add role '%s'. (Error message : %s)" % (role, str(e)) )

    if not errorFlag:
        out.success("Completed added roles setup without errors.")

    out.closeSubsection("Added roles setup")



## _installUpdatedSecurity
# 
# Setup updated security configuration
def _installUpdatedSecurity(self, out, updated_security):
    errorFlag = False

    out.openSubsection("Updated security setup")


    try: # Check for factory tool existance
      url_tool = getToolByName(self, 'portal_url')
      portal_object = url_tool.getPortalObject()
    except AttributeError:
      errorFlag = True
      out.error("'portal_url' tool not found !")
    else:
      for object_name, object_security in updated_security.items(): # For each object to be modified

        # We explode the path to get containers id.
        created_object_exploded_path = object_name.split('/')

        # The root container is the portal.
        container = portal_object

        if object_name != '/':
          # We run through the path to get the container of the object to be created.
          for object_id in created_object_exploded_path:
            # Check if the object with id 'object_id' exist into the current container.
            if not object_id in container.objectIds():
              container = None
              break;
            # If it exist, it become the current container.
            container = getattr(container, object_id, None)
            # We stop if the object is not found.
            if not container:
              break;

        # Whe check if the object to be created is not already present.
        if not container:
          errorFlag = True
          out.error("object '%s' not found." % object_name )
        else:
          for security in object_security:
            try:
              container.manage_permission(**security)
              out.log("Set '%s' security configuration for permission '%s'." % (object_name, security.get('permission_to_manage', '')) )
            except Exception, e:
              errorFlag = True
              out.error("failed '%s' security configuration for permission '%s'. (Error message : %s)" % (object_name, security.get('permission_to_manage', ''), str(e)) )

    if not errorFlag:
        out.success("Completed updated security setup without errors.")

    out.closeSubsection("Updated security setup")



#########################################################################################################
######################################### Remove functions ##############################################
#########################################################################################################



## _removeAddedRoles
# 
# Remove added roles configuration.
# TODO : Create a remove added roles method.
def _removeAddedRoles(self, out, added_roles):

    out.openSubsection("Added roles removal")

    out.info("Added roles configuration can not be removed.")

    out.closeSubsection("Added roles removal")



## _removeUpdatedSecurity
# 
# Remove updated security configuration.
# TODO : Create a remove updated security method.
def _removeUpdatedSecurity(self, out, updated_security):

    out.openSubsection("Updated security removal")

    out.info("Updated security configuration can not be removed.")

    out.closeSubsection("Updated security removal")



#########################################################################################################
######################################### Private functions #############################################
#########################################################################################################

