## File "PloneInstaller/Scripts/tools_setup.py"
## -*- coding: utf-8 -*-
# 
# This file use the configuration described in "Configs/tools.py" to setup the portal tools.

from Products.CMFCore.utils import getToolByName

#########################################################################################################
########################################## Public methods ###############################################
#########################################################################################################



def tools_install(self, out, project_name, globals):
    try:
      tools_module = __import__("Configs.tools",
                                            globals, locals(),
                                           [
                                              'TOOLS',
                                            ]
                                          )

      out.openSection("Configs.tools setup")

      # Tools installed configuration
      tools = getattr(tools_module, 'TOOLS', None)
      if tools != None:
        _installTools(self, out, project_name, tools)

      out.closeSection("Configs.tools setup")

    except ImportError:
        out.info("Configs.tools setup : Noting to be done.")



##---------------------------------------------------------------------------



def tools_remove(self, out, project_name, globals):
    try:
      tools_module = __import__("Configs.tools",
                                            globals, locals(),
                                           [
                                              'TOOLS',
                                            ]
                                          )

      out.openSection("Configs.tools removal")

      # Tools installed configuration
      tools = getattr(tools_module, 'TOOLS', None)
      if tools != None:
        _removeTools(self, out, tools)

      out.closeSection("Configs.tools removal")

    except ImportError:
        out.info("Configs.tools removal : Noting to be done.")



#########################################################################################################
######################################## Install functions ##############################################
#########################################################################################################



## _installTools
# 
# Setup tools by this product
def _installTools(self, out, project_name, tools):
    errorFlag = False

    out.openSubsection("Tools setup")


    try:
        addTool = self.manage_addProduct[project_name].manage_addTool
    except Exception, e:
        errorFlag = True
        out.error("failed to get addTool. (Error message : %s)" % (str(e),) )
    else:
        for tool_description in tools:
            try:
              addTool(tool_description['name'], None)
              out.info("Added %s to the portal root folder.\n" % tool_description['name'])
              out.info("Tool '%s' successfully installed in the portal root." % tool_description['name'])
            except Exception, e:
                errorFlag = True
                out.error("failed to install tool '%s'. (Error message : %s)" % (tool_description['name'], str(e)) )

    if not errorFlag:
        out.success("Completed tools installed setup without errors.")

    out.closeSubsection("Tools setup")



#########################################################################################################
######################################### Remove functions ##############################################
#########################################################################################################



## _removeTools
# 
# Remove tools  configuration.
def _removeTools(self, out, tools):

    out.openSubsection("Tools removal")

    try:
        url_tool = getToolByName(self, 'portal_url')
        portal = url_tool.getPortalObject()
    except Exception, e:
        errorFlag = True
        out.error("failed to get url_tool. (Error message : %s)" % (str(e),) )
    else:
        for tool_description in tools:
            if(tool_description['delete']):
                try:
                    portal.manage_delObjects(ids = [ tool_description['id'], ] )

                    out.info("Tool '%s' successfully removed." % tool_description['name'])
                except Exception, e:
                    errorFlag = True
                    out.error("failed to remove tool '%s'. (Error message : %s)" % (tool_description['name'], str(e)) )

    out.closeSubsection("Tools removal")



#########################################################################################################
######################################### Private functions #############################################
#########################################################################################################

