## File "PloneInstaller/Scripts/types_setup.py"
## -*- coding: utf-8 -*-
# 
# This file use the configuration described in "Configs/types.py" to setup the portal types.

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes

#########################################################################################################
########################################## Public methods ###############################################
#########################################################################################################



def types_install(self, out, project_name, globals):
    try:
      types_module = __import__("Configs.types",
                                            globals, locals(),
                                           [
                                              'TYPES_INSTALLED',
                                              'TYPES_UPDATED_PROPERTIES',
                                              'TYPES_DELETED_ALIASES',
                                              'TYPES_CREATED_ALIASES',
                                              'TYPES_UPDATED_ALIASES',
                                            ]
                                          )

      out.openSection("Configs.types setup")

      # Types installed configuration
      types_installed = getattr(types_module, 'TYPES_INSTALLED', None)
      if types_installed != None:
        _installTypesInstalled(self, out, project_name, types_installed)

      # Types updated properties configuration
      types_updated_properties = getattr(types_module, 'TYPES_UPDATED_PROPERTIES', None)
      if types_updated_properties != None:
        _installTypesUpdatedProperties(self, out, types_updated_properties)

      # Types deleted aliases configuration
      types_deleted_aliases = getattr(types_module, 'TYPES_DELETED_ALIASES', None)
      if types_deleted_aliases != None:
        _installTypesDeletedAliases(self, out, types_deleted_aliases)

      # Types created aliases configuration
      types_created_aliases = getattr(types_module, 'TYPES_CREATED_ALIASES', None)
      if types_created_aliases != None:
        _installTypesCreatedAliases(self, out, types_created_aliases)

      # Types updated aliases configuration
      types_updated_aliases = getattr(types_module, 'TYPES_UPDATED_ALIASES', None)
      if types_updated_aliases != None:
        _installTypesUpdatedAliases(self, out, types_updated_aliases)

      out.closeSection("Configs.types setup")

    except ImportError:
        out.info("Configs.types setup : Noting to be done.")



##---------------------------------------------------------------------------



def types_remove(self, out, project_name, globals):
    try:
      types_module = __import__("Configs.types",
                                            globals, locals(),
                                           [
                                              'TYPES_INSTALLED',
                                              'TYPES_UPDATED_PROPERTIES',
                                              'TYPES_DELETED_ALIASES',
                                              'TYPES_CREATED_ALIASES',
                                              'TYPES_UPDATED_ALIASES',
                                            ]
                                          )

      out.openSection("Configs.types removal")

      # Types installed configuration
      types_installed = getattr(types_module, 'TYPES_INSTALLED', None)
      if types_installed != None:
        _removeTypesInstalled(self, out, types_installed)

      # Types updated properties configuration
      types_updated_properties = getattr(types_module, 'TYPES_UPDATED_PROPERTIES', None)
      if types_updated_properties != None:
        _removeTypesUpdatedProperties(self, out, types_updated_properties)

      # Types deleted aliases configuration
      types_deleted_aliases = getattr(types_module, 'TYPES_DELETED_ALIASES', None)
      if types_deleted_aliases != None:
        _removeTypesDeletedAliases(self, out, types_deleted_aliases)

      # Types created aliases configuration
      types_created_aliases = getattr(types_module, 'TYPES_CREATED_ALIASES', None)
      if types_created_aliases != None:
        _removeTypesCreatedAliases(self, out, types_created_aliases)

      # Types updated aliases configuration
      types_updated_aliases = getattr(types_module, 'TYPES_UPDATED_ALIASES', None)
      if types_updated_aliases != None:
        _removeTypesUpdatedAliases(self, out, types_updated_aliases)

      out.closeSection("Configs.types removal")

    except ImportError:
        out.info("Configs.types removal : Noting to be done.")



#########################################################################################################
######################################## Install functions ##############################################
#########################################################################################################



## _installTypesInstalled
# 
# Setup types installed by this product
def _installTypesInstalled(self, out, project_name, types_installed):
    errorFlag = False

    out.openSubsection("Types installed setup")

    try:
        installTypes(self, out, listTypes(project_name), project_name)
    except Exception, e:
        errorFlag = True
        out.error("failed to install product types. (Error message : %s)" % (str(e),) )

    if not errorFlag:
        out.success("Completed types installed setup without errors.")

    out.closeSubsection("Types installed setup")



##---------------------------------------------------------------------------



## _installTypesUpdatedProperties
# 
# Setup types updated properties configuration
def _installTypesUpdatedProperties(self, out, types_updated_properties):
    errorFlag = False

    out.openSubsection("Types updated properties setup")

    try: # Check for types tool existance
        types_tool = getToolByName(self, 'portal_types')
    except AttributeError:
        out.error("'portal_types' tool not found !")
    else:

        for type_name, type_properties in types_updated_properties.items(): # For each type
            try:
                type_info = types_tool.getTypeInfo(type_name)

                if not type_info:
                    raise Exception, "type does not exist."

                # We check for list properties which new values can be appended to existings ones.
                for property_name, property_value in type_properties.items():
                    if isinstance(property_value, dict): # Test si la valeur est un dictionnaire.
                        if not (property_value.has_key("values") and property_value.has_key("append")): # Test si le dictionnaire est ok.
                            raise Exception, "property '%s' is an invalid dictionnary." % property_name

                        if not property_value.get("append", False):
                            type_properties[property_name] = property_value.get("values", [])
                        else:
                            actual_value = type_info.getProperty(property_name)
                            if actual_value == None:
                                raise Exception, "property '%s' is not defined." % property_name

                            if not isinstance(actual_value, tuple):
                                raise Exception, "property '%s' is not a list." % property_name

                            actual_value = list(actual_value)

                            actual_value.extend( [value for value in property_value.get("values", []) if not value in actual_value ] )

                            type_properties[property_name] = actual_value

                # We updated properties values
                type_info.manage_changeProperties(**type_properties)
                out.log("Updated type '%s' properties." % type_name)

            except Exception, e:
                errorFlag = True
                out.error("failed to update type '%s' properties. (Error message : %s)" % (type_name, str(e)) )

        if not errorFlag:
            out.success("Completed types updated properties setup without errors.")

    out.closeSubsection("Types updated properties setup")



##---------------------------------------------------------------------------



## _installTypesDeletedAliases
# 
# Setup types deleted aliases configuration
def _installTypesDeletedAliases(self, out, types_deleted_aliases):
    errorFlag = False

    out.openSubsection("Types deleted aliases setup")

    try: # Check for types tool existance
        types_tool = getToolByName(self, 'portal_types')
    except AttributeError:
        out.error("'portal_types' tool not found !")
    else:

        for type_name, type_aliases in types_deleted_aliases.items(): # For each type
            try:
                type_info = types_tool.getTypeInfo(type_name)

                if not type_info:
                    raise Exception, "type does not exist."

                # We get the list of existing aliases.
                current_aliases = type_info.getMethodAliases()

                for alias in type_aliases:
                  if current_aliases.has_key(alias) :
                      del current_aliases[alias]
                  else :
                      errorFlag = True
                      out.error("Type '%s' has no alias named '%s'." % (type_name, alias) )

                # We delete aliases values
                type_info.setMethodAliases(current_aliases)
                out.log("Deleted type '%s' aliases." % type_name)

            except Exception, e:
                errorFlag = True
                out.error("failed to delete type '%s' aliases. (Error message : %s)" % (type_name, str(e)) )

        if not errorFlag:
            out.success("Completed types deleted aliases setup without errors.")

    out.closeSubsection("Types deleted aliases setup")



##---------------------------------------------------------------------------



## _installTypesCreatedAliases
# 
# Setup types created aliases configuration
def _installTypesCreatedAliases(self, out, types_created_aliases):
    errorFlag = False

    out.openSubsection("Types created aliases setup")

    try: # Check for types tool existance
        types_tool = getToolByName(self, 'portal_types')
    except AttributeError:
        out.error("'portal_types' tool not found !")
    else:

        for type_name, type_aliases in types_created_aliases.items(): # For each type
            try:
                type_info = types_tool.getTypeInfo(type_name)

                if not type_info:
                    raise Exception, "type does not exist."

                # We get the list of existing aliases.
                current_aliases = type_info.getMethodAliases()

                for alias_name, alias_value in type_aliases.items():
                  if not current_aliases.has_key(alias_name) :
                      current_aliases[alias_name] = alias_value
                  else :
                      errorFlag = True
                      out.error("Type '%s' has already alias named '%s'." % (type_name, alias_name) )

                # We delete aliases values
                type_info.setMethodAliases(current_aliases)
                out.log("Created type '%s' aliases." % type_name)

            except Exception, e:
                errorFlag = True
                out.error("failed to create type '%s' aliases. (Error message : %s)" % (type_name, str(e)) )

        if not errorFlag:
            out.success("Completed types created aliases setup without errors.")

    out.closeSubsection("Types created aliases setup")



##---------------------------------------------------------------------------



## _installTypesUpdatedAliases
# 
# Setup types updated aliases configuration
def _installTypesUpdatedAliases(self, out, types_updated_aliases):
    errorFlag = False

    out.openSubsection("Types updated aliases setup")

    try: # Check for types tool existance
        types_tool = getToolByName(self, 'portal_types')
    except AttributeError:
        out.error("'portal_types' tool not found !")
    else:

        for type_name, type_aliases in types_updated_aliases.items(): # For each type
            try:
                type_info = types_tool.getTypeInfo(type_name)

                if not type_info:
                    raise Exception, "type does not exist."

                # We get the list of existing aliases.
                current_aliases = type_info.getMethodAliases()

                for alias_name, alias_value in type_aliases.items():
                  if current_aliases.has_key(alias_name) :
                      current_aliases[alias_name] = alias_value
                  else :
                      errorFlag = True
                      out.error("Type '%s' has no alias named '%s'." % (type_name, alias_name) )

                # We delete aliases values
                type_info.setMethodAliases(current_aliases)
                out.log("Updated type '%s' aliases." % type_name)

            except Exception, e:
                errorFlag = True
                out.error("failed to update type '%s' aliases. (Error message : %s)" % (type_name, str(e)) )

        if not errorFlag:
            out.success("Completed types updated aliases setup without errors.")

    out.closeSubsection("Types updated aliases setup")



#########################################################################################################
######################################### Remove functions ##############################################
#########################################################################################################



## _removeTypesInstalled
# 
# Remove types installed configuration.
# TODO : Create a remove types installed method.
def _removeTypesInstalled(self, out, types_installed):

    out.openSubsection("Types installed removal")

    try: # Check for types tool existance
        types_tool = getToolByName(self, 'portal_types')
    except AttributeError:
        out.error("'portal_types' tool not found !")
    else:
        out.info("Types installed configuration can not be removed.")

    out.closeSubsection("Types installed removal")



##---------------------------------------------------------------------------



## _removeTypesUpdatedProperties
# 
# Remove types updated properties configuration.
# TODO : Create a remove types updated properties method.
def _removeTypesUpdatedProperties(self, out, types_updated_properties):

    out.openSubsection("Types updated properties removal")

    try: # Check for types tool existance
        types_tool = getToolByName(self, 'portal_types')
    except AttributeError:
        out.error("'portal_types' tool not found !")
    else:
        out.info("Types updated properties configuration can not be removed.")

    out.closeSubsection("Types updated properties removal")



##---------------------------------------------------------------------------



## _removeTypesDeletedAliases
# 
# Remove types deleted aliases configuration.
# TODO : Create a remove types deleted aliases method.
def _removeTypesDeletedAliases(self, out, types_deleted_aliases):

    out.openSubsection("Types deleted aliases removal")

    try: # Check for types tool existance
        types_tool = getToolByName(self, 'portal_types')
    except AttributeError:
        out.error("'portal_types' tool not found !")
    else:
        out.info("Types deleted aliases configuration can not be removed.")

    out.closeSubsection("Types deleted aliases removal")



##---------------------------------------------------------------------------



## _removeTypesCreatedAliases
# 
# Setup types Created aliases configuration
def _removeTypesCreatedAliases(self, out, types_created_aliases):
    errorFlag = False

    out.openSubsection("Types created aliases removal")

    try: # Check for types tool existance
        types_tool = getToolByName(self, 'portal_types')
    except AttributeError:
        out.error("'portal_types' tool not found !")
    else:

        for type_name, type_aliases in types_created_aliases.items(): # For each type
            try:
                type_info = types_tool.getTypeInfo(type_name)

                if not type_info:
                    raise Exception, "type does not exist."

                # We get the list of existing aliases.
                current_aliases = type_info.getMethodAliases()

                for alias in type_aliases:
                  if current_aliases.has_key(alias) :
                      del current_aliases[alias]
                  else :
                      errorFlag = True
                      out.error("Type '%s' has no alias named '%s'." % (type_name, alias) )

                # We delete aliases values
                type_info.setMethodAliases(current_aliases)
                out.log("Removed type '%s' create aliases." % type_name)

            except Exception, e:
                errorFlag = True
                out.error("failed to remove type '%s' created aliases. (Error message : %s)" % (type_name, str(e)) )

        if not errorFlag:
            out.success("Completed types created aliases setup without errors.")

    out.closeSubsection("Types created aliases removal")



##---------------------------------------------------------------------------



## _removeTypesUpdatedAliases
# 
# Remove types updated aliases configuration.
# TODO : Create a remove types updated aliases method.
def _removeTypesUpdatedAliases(self, out, types_updated_aliases):

    out.openSubsection("Types updated aliases removal")

    try: # Check for types tool existance
        types_tool = getToolByName(self, 'portal_types')
    except AttributeError:
        out.error("'portal_types' tool not found !")
    else:
        out.info("Types updated aliases configuration can not be removed.")

    out.closeSubsection("Types updated aliases removal")



#########################################################################################################
######################################### Private functions #############################################
#########################################################################################################

